//finite state machine on serial port
//BUTTON, SEND(1), SEND(2)

module serialfsm(REC,SEND,CLK,LED,BUTTON,MOSI,MISO,DAC_CS,DAC_CLR,SCK,
	SPI_SS_B,AMP_CS,AD_CONV,SF_CE0,FPGA_INIT_B,AMP_SHDN);

	input REC;
	output SEND;
    input CLK;
	input BUTTON;
	output [7:0]LED;
	output MOSI;
	input MISO;
	output DAC_CS;
	output SCK;
	output DAC_CLR;
	output SPI_SS_B;
	output AMP_CS;
	output AD_CONV;
	output SF_CE0;
	output FPGA_INIT_B;
	output AMP_SHDN;

	//internal connections
	wire rsclk;
	wire fsmclk;
	wire addaclk;
	wire [7:0]command;
	wire rcvd;
	wire spircvd;
	wire tr_inproc;
	wire cs;
	wire [31:0]adc_data;
	reg [11:0]spidata;
	//state registers
	reg [7:0]tosend=0;
	reg trmt=0;
	reg rcv=0;
	reg spitrmt=0;
	reg [7:0]state=0;
	reg [1:0]rcsh=0;
	reg [15:0]delay;	//delay for button routine
	reg sel_dac;		//1->SPI data for DAC else for amplifier

	prescaler p1(CLK,rsclk,fsmclk,addaclk);
	receiver r1(REC,command,rsclk,rcvd);
	transmitter t1(rsclk,tosend,trmt,SEND,tr_inproc);
	spitrans spi1(addaclk,spidata,spitrmt,MOSI,cs,sel_dac);
	spirec spi2(addaclk,adc_data,rcv,spircvd,MISO,AD_CONV);

	//command interpreter
	always@(posedge fsmclk)
		begin
		rcsh[0]<=rcvd;
		rcsh[1]<=rcsh[0];
		
		case(state)
		8'h0:
			begin
			rcv<=0;
			if(BUTTON)state<=8'h13;
			if(rcsh==2'b01)
				case(command)
				8'h01:state<=8'h1;		//send ATOM
				8'h02:state<=8'h9;		//send HELLO
				8'h03:state<=8'h1B;		//SPI to DAC
				8'h04:state<=8'h1E;		//SPI to amlifier
				8'h05:state<=8'h20;		//SPI from ADC
				8'hFA:state<=8'h29;		//Connecting to device
				8'h10:state<=8'h35;		//start AD to DA
				default:state<=8'h0;
				endcase
			end
//ATOM
		8'h1:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h41;
				trmt=1;
				state<=8'h2;
				end
			end
		8'h2:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h3; end
			end
		8'h3:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h54;
				trmt=1;
				state<=8'h4;
				end
			end
		8'h4:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h5; end
			end
		8'h5:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h4F;
				trmt=1;
				state<=8'h6;
				end
			end
		8'h6:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h7; end
			end
		8'h7:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h4D;
				trmt=1;
				state<=8'h8;
				end
			end
		8'h8:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h0; end
			end
//HELLO
		8'h9:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h48;
				trmt=1;
				state<=8'hA;
				end
			end
		8'hA:
			begin
			if(tr_inproc)begin trmt=0;state<=8'hB; end
			end
		8'hB:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h45;
				trmt=1;
				state<=8'hC;
				end
			end
		8'hC:
			begin
			if(tr_inproc)begin trmt=0;state<=8'hD; end
			end
		8'hD:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h4C;
				trmt=1;
				state<=8'hE;
				end
			end
		8'hE:
			begin
			if(tr_inproc)begin trmt=0;state<=8'hF; end
			end
		8'hF:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h4C;
				trmt=1;
				state<=8'h10;
				end
			end
		8'h10:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h11; end
			end
		8'h11:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h4F;
				trmt=1;
				state<=8'h12;
				end
			end
		8'h12:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h0; end
			end
//BUMM
		8'h13:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h42;
				trmt=1;
				state<=8'h14;
				end
			end
		8'h14:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h15; end
			end
		8'h15:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h55;
				trmt=1;
				state<=8'h16;
				end
			end
		8'h16:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h17; end
			end
		8'h17:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'h4D;
				trmt=1;
				state<=8'h18;
				end
			end
		8'h18:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h19; end
			end
		8'h19:
			begin
			delay<=16'h0;
			if(!tr_inproc)
				begin
				tosend<=8'h4D;
				trmt=1;
				state<=8'h1A;
				end
			end
		8'h1A:
			begin
			trmt=0;
			delay<=delay+1;
			if(delay==16'hFFFF)state<=8'h00;
			end
//SPI transfer to DAC
		8'h1B:
			begin
			spitrmt<=0;
			sel_dac<=1;
			if(rcsh==2'b01)
				begin
				spidata[7:0]<=command[7:0];
				state<=8'h1C;
				end	
			end
		8'h1C:
			begin
			if(rcsh==2'b01)
				begin
				spidata[11:8]<=command[3:0];
				state<=8'h1D;
				end	
			end
		8'h1D:
			begin
			spitrmt<=1;
			state<=8'h00;
			end
//SPI transfer to programmable preamplifier (27h-ra a legtisztbb)
		8'h1E:
			begin
			spitrmt<=0;
			sel_dac<=0;
			if(rcsh==2'b01)
				begin
				spidata[7:0]<=command[7:0];
				state<=8'h1F;
				end
			end
		8'h1F:
			begin
			spitrmt<=1;
			state<=8'h00;
			end
//SPI reception from ADC
		8'h20:
			begin
			rcv<=1;
			state<=8'h21;
			end
		8'h21:
			begin
			rcv<=0;
			if(!tr_inproc)
				begin
				tosend<=adc_data[7:0];
				trmt=1;
				state<=8'h22;
				end
			end
		8'h22:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h23; end
			end
		8'h23:
			begin
			if(!tr_inproc)
					begin
					tosend<=adc_data[15:8];
					trmt=1;
					state<=8'h24;
					end
			end
		8'h24:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h25; end
			end
		8'h25:
			begin
			if(!tr_inproc)
					begin
					tosend<=adc_data[23:16];
					trmt=1;
					state<=8'h26;
					end
			end
		8'h26:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h27; end
			end
		8'h27:
			begin
			if(!tr_inproc)
					begin
					tosend<=adc_data[31:24];
					trmt=1;
					state<=8'h28;
					end
			end
		8'h28:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h00; end
			end
//Connecting to device
		8'h29:
			begin
			if(!tr_inproc)
				begin
				tosend<=8'hFA;
				trmt=1;
				state<=8'h30;
				end
			end
		8'h30:
			begin
			if(tr_inproc)begin trmt=0;state<=8'h00; end
			end
//start AD to DA
		8'h35:
			begin
			rcv<=1;
			if(rcsh==2'b01)state<=8'h00;
			else state<=8'h36;
			end
		8'h36:
			begin
			rcv<=0;
			spidata[11:0]<=adc_data[13:2];
			spitrmt<=0;
			sel_dac<=1;
			if(rcsh==2'b01)state<=8'h00;
			else state<=8'h37;
			end
		8'h37:
			begin
			spitrmt<=1;
			if(rcsh==2'b01)state<=8'h00;
			else state<=8'h38;
			end
		8'h38:
			begin
			if(rcsh==2'b01)state<=8'h00;
			else state<=8'h35;
			end

		default:state<=4'h0;
		endcase
		end

	assign LED=command;
	assign SCK=addaclk;
	assign DAC_CLR=1;
	assign DAC_CS=cs|!sel_dac;

	assign SPI_SS_B=1;
	assign AMP_CS=cs|sel_dac;
	assign SF_CE0=1;
	assign FPGA_INIT_B=1;
	assign AMP_SHDN=0;

endmodule
